<?php
class DB
{
	public static function CurrentConnection()
	{
		global $currentConnection;
		return $currentConnection ? $currentConnection : DB::DefaultConnection();
	}

	public static function CurrentConnectionId()
	{
		$conn = DB::CurrentConnection();
		return $conn->connId;
	}

	public static function DefaultConnection()
	{
		global $cman;
		return $cman->getDefault();
	}

	public static function ConnectionByTable( $table )
	{
		global $cman;
		return $cman->byTable($table);
	}

	public static function ConnectionByName( $name )
	{
		global $cman;
		return $cman->byName( $name );
	}

	public static function SetConnection( $connection )
	{
		global $currentConnection;
		if ( is_string( $connection ) )
		{
			$currentConnection = DB::ConnectionByName( $connection );
		}
		else if ( is_a($connection, 'Connection') ) {
		 	$currentConnection = $connection;
		}
	}

	public static function LastId()
	{
		return DB::CurrentConnection()->getInsertedId();
	}

	public static function Query( $sql )
	{
		return DB::CurrentConnection()->querySilent( $sql );
	}

	public static function Exec( $sql )
	{
		return DB::CurrentConnection()->execSilent( $sql ) != NULL;
	}

	public static function LastError()
	{
		return DB::CurrentConnection()->lastError();
	}

	public static function Select($table, $userConditions = array() )
	{
		$tableInfo = DB::_getTableInfo($table);

		if ( !$tableInfo )
			return false;

		$whereSql = DB::_getWhereSql($userConditions, $tableInfo["fields"]);

		$sql = "SELECT * FROM ".DB::CurrentConnection()->addTableWrappers( $tableInfo['fullName'] ) . $whereSql;
		$queryResult = DB::CurrentConnection()->querySilent( $sql );

		return $queryResult;
	}

	public static function Delete($table, $userConditions = array() )
	{
		$tableInfo = DB::_getTableInfo($table);

		if ( !$tableInfo )
			return false;

		$whereSql = DB::_getWhereSql($userConditions, $tableInfo["fields"]);

		if( $whereSql == "" )
			return false;

		$sql = "DELETE FROM ".DB::CurrentConnection()->addTableWrappers( $tableInfo['fullName'] ) . $whereSql;
		$ret = DB::CurrentConnection()->execSilent( $sql );

		return $ret;
	}

	public static function Insert($table, $data)
	{
		$result = false;
		$tableInfo = DB::_getTableInfo($table);

		if ( !$tableInfo )
			return false;

		$iFields = "";
		$iValues = "";
		$blobs = array();
		foreach($data as $fieldname => $value)
		{
			$field = getArrayElementNC($tableInfo["fields"], $fieldname);

			// user field not found in table
			if ( is_null($field) )
				continue;

			$iFields.= DB::CurrentConnection()->addFieldWrappers( $field["name"] ).",";
			$iValues.= DB::_prepareValue($value, $field["type"]) . ",";

			if( DB::CurrentConnection()->dbType == nDATABASE_Oracle || DB::CurrentConnection()->dbType == nDATABASE_DB2 || DB::CurrentConnection()->dbType == nDATABASE_Informix )
			{
				if( IsBinaryType( $field["type"] ) )
					$blobs[ $field["name"] ] = $value;

				if( DB::CurrentConnection()->dbType == nDATABASE_Informix && IsTextType( $field["type"] ) )
					$blobs[ $field["name"] ] = $value;
			}
		}

		if( $iFields != "" && $iValues != "" )
		{
			$iFields = substr($iFields, 0, -1);
			$iValues = substr($iValues, 0, -1);
			$sql = "INSERT INTO ".DB::CurrentConnection()->addTableWrappers( $tableInfo['fullName'] )." (".$iFields.") values (".$iValues.")";

			if ( count($blobs) > 0 )
				$result = DB::_execSilentWithBlobProcessing($blobs, $sql, $tableInfo['fields']);
			else
				$result = DB::CurrentConnection()->execSilent( $sql );
		}

		return $result;
	}

	public static function Update($table, $data, $userConditions)
	{
		$result = false;
		$tableInfo = DB::_getTableInfo($table);

		if ( !$tableInfo )
			return false;

		$whereSql = DB::_getWhereSql($userConditions, $tableInfo["fields"]);

		if( $whereSql == "" )
			return false;

		$updateValues = array();
		$blobs = array();
		foreach( $data as $fieldname => $value )
		{
			$field = getArrayElementNC($tableInfo["fields"], $fieldname);

			// user field not found in table
			if ( is_null($field) )
				continue;

			$prepareFieldName = DB::CurrentConnection()->addFieldWrappers( $field["name"] );
			$prepareValue = DB::_prepareValue($value, $field["type"]);
			$updateValues[] = $prepareFieldName."=".$prepareValue;

			if ( DB::CurrentConnection()->dbType == nDATABASE_Oracle || DB::CurrentConnection()->dbType == nDATABASE_DB2 || DB::CurrentConnection()->dbType == nDATABASE_Informix )
			{
				if ( IsBinaryType( $field["type"] ) )
					$blobs[ $field["name"] ] = $value;

				if ( DB::CurrentConnection()->dbType == nDATABASE_Informix && IsTextType( $field["type"] ) )
					$blobs[ $field["name"] ] = $value;
			}
		}

		if ( count($updateValues) > 0 )
		{
			$updateSQL = implode(",", $updateValues);
			$sql = "UPDATE ".DB::CurrentConnection()->addTableWrappers( $tableInfo['fullName'] )." SET ". $updateSQL . $whereSql;

			if ( count($blobs) > 0 )
				$result = DB::_execSilentWithBlobProcessing($blobs, $sql, $tableInfo['fields']);
			else
				$result = DB::CurrentConnection()->execSilent( $sql );
		}

		return $result;
	}

	protected static function _getWhereSql($userConditions, $founedfields)
	{
		if( !is_array( $userConditions ) )
		{
			$whereSql = trim( $userConditions );
			if( $whereSql != "")
				$whereSql = " WHERE " . $whereSql;
			return $whereSql;
		}

		$conditions = array();
		foreach($userConditions as $fieldname => $value)
		{
			$field = getArrayElementNC($founedfields, $fieldname);
			// user field not found in table
			if ( is_null($field) )
				continue;

			$wrappedField = DB::CurrentConnection()->addFieldWrappers( $field["name"] );
			if ( is_null($value) )
			{
				$conditions[] = $wrappedField . " IS NULL";
			}
			else
			{
				$conditions[] = $wrappedField . "=" . DB::_prepareValue($value, $field["type"]);
			}
		}

		$whereSql = "";
		if( count($conditions) > 0 )
		{
			$whereSql .= " WHERE " . implode(" AND ", $conditions);
		}

		return $whereSql;
	}

	/**
	 * @param Array blobs
	 * @param String dalSQL
	 * @param Array tableinfo
	 */
	protected static function _execSilentWithBlobProcessing($blobs, $dalSQL, $tableinfo)
	{
		$blobTypes = array();
		if( DB::CurrentConnection()->dbType == nDATABASE_Informix )
		{
			foreach( $blobs as $fname => $fvalue )
			{
				$blobTypes[ $fname ] = $tableinfo[ $fname ]["type"];
			}
		}

		DB::CurrentConnection()->execSilentWithBlobProcessing( $dalSQL, $blobs, $blobTypes );
	}

	protected static function _prepareValue($value, $type)
	{
		if ( is_null($value) )
			return "NULL";

		if( DB::CurrentConnection()->dbType == nDATABASE_Oracle || DB::CurrentConnection()->dbType == nDATABASE_DB2 || DB::CurrentConnection()->dbType == nDATABASE_Informix )
		{
			if( IsBinaryType($type) )
			{
				if( DB::CurrentConnection()->dbType == nDATABASE_Oracle )
					return "EMPTY_BLOB()";

				return "?";
			}

			if( DB::CurrentConnection()->dbType == nDATABASE_Informix  && IsTextType($type) )
				return "?";
		}

		if( IsNumberType($type) && !is_numeric($value) )
		{
			$value = trim($value);
			$value = str_replace(",", ".", $value);
			if ( !is_numeric($value) )
				return "NULL";
		}

		if( IsDateFieldType($type) || IsTimeType($type) )
		{
			if( !$value )
				return "NULL";

			// timestamp
			if ( is_int($value) )
			{
				if ( IsDateFieldType($type) )
				{
					$value = getYMDdate($value) . " " . getHISdate($value);
				}
				else if ( IsTimeType($type) )
				{
					$value = getHISdate($value);
				}
			}

			return DB::CurrentConnection()->addDateQuotes( $value );
		}

		if( NeedQuotes($type) )
			return DB::CurrentConnection()->prepareString( $value );

		return $value;
	}

	/**
	 * 	Find table info stored in the project file
	 *
	 */
	public static function _findDalTable( $table, $conn = null )
	{
		global $dalTables;
		if( !$conn )
			$conn = DB::CurrentConnection();
		$tableName = $conn->getTableNameComponents( $table );

		DB::_fillTablesList( $conn );

		//	exact match
		foreach( $dalTables[$conn->connId] as $t )
		{
			if( $t["schema"] == $tableName["schema"] && $t["name"] == $tableName["table"] )
				return $t;
		}

		//	case-insensitive
		$tableName["schema"] = strtoupper( $tableName["schema"] );
		$tableName["table"] = strtoupper( $tableName["table"] );

		foreach( $dalTables[$conn->connId] as $t )
		{
			if( strtoupper( $t["schema"] ) == $tableName["schema"] && strtoupper( $t["name"] ) == $tableName["table"] )
				return $t;
		}
		return null;
	}

	/**
	 * 	Get list of table field names and types
	 *	Check tables stored in the project first, then fetch it from the database.
	 *
	 */
	public static function _getTableInfo($table, $connId = null )
	{
		global $dal_info, $tableinfo_cache, $cman;
		if( !$connId )
			$connId = DB::CurrentConnectionId();

		//	prepare cache
		if( !isset($tableinfo_cache[ $connId ] ) )
			$tableinfo_cache[ $connId ] = array();

		$tableInfo = array();


		$tableDescriptor = DB::_findDalTable( $table, $cman->byId( $connId ) );

		if ( $tableDescriptor )
		{
			importTableInfo( $tableDescriptor["varname"] );

			$tableInfo["fields"] = $dal_info[ $tableDescriptor["varname"] ];

			if( $tableDescriptor["schema"] )
				$tableInfo["fullName"] = $tableDescriptor["schema"] . "." . $tableDescriptor["name"];
			else
				$tableInfo["fullName"] = $tableDescriptor["name"];
		}
		else
		{
			//	check cache first
			if( isset($tableinfo_cache[ $connId ][ $table ] ) )
				return $tableinfo_cache[ $connId ][ $table ];

			//	fetch table info from the database
			$helpSql = "select * from " . DB::CurrentConnection()->addTableWrappers( $table ) . " where 1=0";

			$tableInfo["fullName"] = $table;
			$tableInfo["fields"] = array();

			// in case getFieldsList throws error
			$tableinfo_cache[ $connId ][ $table ] = false;

			$fieldList = DB::CurrentConnection()->getFieldsList($helpSql);
			foreach ($fieldList as $f )
			{
				$tableInfo["fields"][ $f["fieldname"] ] = array( "type" => $f["type"], "name" => $f["fieldname"] );
			}
			$tableinfo_cache[ $connId ][ $table ] = $tableInfo;
		}

		return $tableInfo;
	}


	protected static function _fillTablesList( $conn )
	{
		global $dalTables;
		if( !$conn )
			$conn = DB::CurrentConnection();
		if( $dalTables[ $conn->connId ] )
			return;
		$dalTables[ $conn->connId ] = array();
		if( "erp_jarcgroupcos_at_localhost" == $conn->connId )
		{
			$dalTables[$conn->connId][] = array("name" => "activity_log", "varname" => "erp_jarcgroupcos_at_localhost__activity_log", "altvarname" => "activity_log", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "announcement", "varname" => "erp_jarcgroupcos_at_localhost__announcement", "altvarname" => "announcement", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "applicant_requirements", "varname" => "erp_jarcgroupcos_at_localhost__applicant_requirements", "altvarname" => "applicant_requirements", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "approval_alerts", "varname" => "erp_jarcgroupcos_at_localhost__approval_alerts", "altvarname" => "approval_alerts", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "attendance", "varname" => "erp_jarcgroupcos_at_localhost__attendance", "altvarname" => "attendance", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "attendance_report", "varname" => "erp_jarcgroupcos_at_localhost__attendance_report", "altvarname" => "attendance_report", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "attendance_subordinate", "varname" => "erp_jarcgroupcos_at_localhost__attendance_subordinate", "altvarname" => "attendance_subordinate", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "attendance_type", "varname" => "erp_jarcgroupcos_at_localhost__attendance_type", "altvarname" => "attendance_type", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "basic_system_audit", "varname" => "erp_jarcgroupcos_at_localhost__basic_system_audit", "altvarname" => "basic_system_audit", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "business_partner", "varname" => "erp_jarcgroupcos_at_localhost__business_partner", "altvarname" => "business_partner", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "change_time", "varname" => "erp_jarcgroupcos_at_localhost__change_time", "altvarname" => "change_time", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "chart_of_accounts", "varname" => "erp_jarcgroupcos_at_localhost__chart_of_accounts", "altvarname" => "chart_of_accounts", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "cost_center", "varname" => "erp_jarcgroupcos_at_localhost__cost_center", "altvarname" => "cost_center", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "credit_terms", "varname" => "erp_jarcgroupcos_at_localhost__credit_terms", "altvarname" => "credit_terms", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "cto", "varname" => "erp_jarcgroupcos_at_localhost__cto", "altvarname" => "cto", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "currency", "varname" => "erp_jarcgroupcos_at_localhost__currency", "altvarname" => "currency", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "currency_exchange_rate", "varname" => "erp_jarcgroupcos_at_localhost__currency_exchange_rate", "altvarname" => "currency_exchange_rate", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "department", "varname" => "erp_jarcgroupcos_at_localhost__department", "altvarname" => "department", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "discounts", "varname" => "erp_jarcgroupcos_at_localhost__discounts", "altvarname" => "discounts", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "dtr_project", "varname" => "erp_jarcgroupcos_at_localhost__dtr_project", "altvarname" => "dtr_project", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "employee_dependents", "varname" => "erp_jarcgroupcos_at_localhost__employee_dependents", "altvarname" => "employee_dependents", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "employee_project", "varname" => "erp_jarcgroupcos_at_localhost__employee_project", "altvarname" => "employee_project", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "employee_project_detail", "varname" => "erp_jarcgroupcos_at_localhost__employee_project_detail", "altvarname" => "employee_project_detail", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "employee_project_task", "varname" => "erp_jarcgroupcos_at_localhost__employee_project_task", "altvarname" => "employee_project_task", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "employee_standardv104_uggroups", "varname" => "erp_jarcgroupcos_at_localhost__employee_standardv104_uggroups", "altvarname" => "employee_standardv104_uggroups", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "employee_standardv104_ugmembers", "varname" => "erp_jarcgroupcos_at_localhost__employee_standardv104_ugmembers", "altvarname" => "employee_standardv104_ugmembers", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "employee_standardv104_ugrights", "varname" => "erp_jarcgroupcos_at_localhost__employee_standardv104_ugrights", "altvarname" => "employee_standardv104_ugrights", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "employees", "varname" => "erp_jarcgroupcos_at_localhost__employees", "altvarname" => "employees", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "employment_history", "varname" => "erp_jarcgroupcos_at_localhost__employment_history", "altvarname" => "employment_history", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "fieldwork", "varname" => "erp_jarcgroupcos_at_localhost__fieldwork", "altvarname" => "fieldwork", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "flexben", "varname" => "erp_jarcgroupcos_at_localhost__flexben", "altvarname" => "flexben", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "flexben_type", "varname" => "erp_jarcgroupcos_at_localhost__flexben_type", "altvarname" => "flexben_type", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "holiday", "varname" => "erp_jarcgroupcos_at_localhost__holiday", "altvarname" => "holiday", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "item", "varname" => "erp_jarcgroupcos_at_localhost__item", "altvarname" => "item", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "job_grade", "varname" => "erp_jarcgroupcos_at_localhost__job_grade", "altvarname" => "job_grade", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "leave", "varname" => "erp_jarcgroupcos_at_localhost__leave", "altvarname" => "leave", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "level", "varname" => "erp_jarcgroupcos_at_localhost__level", "altvarname" => "level", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "loan", "varname" => "erp_jarcgroupcos_at_localhost__loan", "altvarname" => "loan", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "loan__detail", "varname" => "erp_jarcgroupcos_at_localhost__loan__detail", "altvarname" => "loan__detail", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "location", "varname" => "erp_jarcgroupcos_at_localhost__location", "altvarname" => "location", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "lot", "varname" => "erp_jarcgroupcos_at_localhost__lot", "altvarname" => "lot", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "missing_logs", "varname" => "erp_jarcgroupcos_at_localhost__missing_logs", "altvarname" => "missing_logs", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "offsetting", "varname" => "erp_jarcgroupcos_at_localhost__offsetting", "altvarname" => "offsetting", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "over_time", "varname" => "erp_jarcgroupcos_at_localhost__over_time", "altvarname" => "over_time", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "payroll_account_type", "varname" => "erp_jarcgroupcos_at_localhost__payroll_account_type", "altvarname" => "payroll_account_type", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "payroll_item", "varname" => "erp_jarcgroupcos_at_localhost__payroll_item", "altvarname" => "payroll_item", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "payroll_leave_report", "varname" => "erp_jarcgroupcos_at_localhost__payroll_leave_report", "altvarname" => "payroll_leave_report", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "payroll_loan", "varname" => "erp_jarcgroupcos_at_localhost__payroll_loan", "altvarname" => "payroll_loan", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "payroll_report_center", "varname" => "erp_jarcgroupcos_at_localhost__payroll_report_center", "altvarname" => "payroll_report_center", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "payroll_system_user", "varname" => "erp_jarcgroupcos_at_localhost__payroll_system_user", "altvarname" => "payroll_system_user", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "payroll_terms", "varname" => "erp_jarcgroupcos_at_localhost__payroll_terms", "altvarname" => "payroll_terms", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "payroll_ticketing", "varname" => "erp_jarcgroupcos_at_localhost__payroll_ticketing", "altvarname" => "payroll_ticketing", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "phase", "varname" => "erp_jarcgroupcos_at_localhost__phase", "altvarname" => "phase", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "process_payroll", "varname" => "erp_jarcgroupcos_at_localhost__process_payroll", "altvarname" => "process_payroll", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "process_payroll_detail", "varname" => "erp_jarcgroupcos_at_localhost__process_payroll_detail", "altvarname" => "process_payroll_detail", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_request", "varname" => "erp_jarcgroupcos_at_localhost__purchase_request", "altvarname" => "purchase_request", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_request_detail", "varname" => "erp_jarcgroupcos_at_localhost__purchase_request_detail", "altvarname" => "purchase_request_detail", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "reimbursement", "varname" => "erp_jarcgroupcos_at_localhost__reimbursement", "altvarname" => "reimbursement", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "reimbursement_detail", "varname" => "erp_jarcgroupcos_at_localhost__reimbursement_detail", "altvarname" => "reimbursement_detail", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "request_for_payment", "varname" => "erp_jarcgroupcos_at_localhost__request_for_payment", "altvarname" => "request_for_payment", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "request_for_payment_detail", "varname" => "erp_jarcgroupcos_at_localhost__request_for_payment_detail", "altvarname" => "request_for_payment_detail", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "sales_order", "varname" => "erp_jarcgroupcos_at_localhost__sales_order", "altvarname" => "sales_order", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "schedule", "varname" => "erp_jarcgroupcos_at_localhost__schedule", "altvarname" => "schedule", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "serial_no", "varname" => "erp_jarcgroupcos_at_localhost__serial_no", "altvarname" => "serial_no", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "tasks", "varname" => "erp_jarcgroupcos_at_localhost__tasks", "altvarname" => "tasks", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "tranasactions_for_approval", "varname" => "erp_jarcgroupcos_at_localhost__tranasactions_for_approval", "altvarname" => "tranasactions_for_approval", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "undertime", "varname" => "erp_jarcgroupcos_at_localhost__undertime", "altvarname" => "undertime", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "uom", "varname" => "erp_jarcgroupcos_at_localhost__uom", "altvarname" => "uom", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "vat_rate", "varname" => "erp_jarcgroupcos_at_localhost__vat_rate", "altvarname" => "vat_rate", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "vat_type", "varname" => "erp_jarcgroupcos_at_localhost__vat_type", "altvarname" => "vat_type", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "violation", "varname" => "erp_jarcgroupcos_at_localhost__violation", "altvarname" => "violation", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
			$dalTables[$conn->connId][] = array("name" => "year_end_pto_conversion", "varname" => "erp_jarcgroupcos_at_localhost__year_end_pto_conversion", "altvarname" => "year_end_pto_conversion", "connId" => "erp_jarcgroupcos_at_localhost", "schema" => "", "connName" => "xynix_main_database at localhost");
		}
	}


	public static function PrepareSQL($sql)
	{
		$args = func_get_args();

		$conn = DB::CurrentConnection();

		$tokens = DB::scanTokenString($sql);

		$replacements = array();
		// build array of replacements in this format:
		//	"offset" => position in the string where replacement should be done
		//  "len" => length of original substring to cut out
		//  "insert" => string to insert in place of cut out

		foreach ($tokens["matches"] as $i => $match) {
			$offset = $tokens["offsets"][$i];
			$token = $tokens["tokens"][$i];

			$repl = array(
				"offset" => $offset,
				"len" => strlen($match)
			);

			$val = "";
			if (is_numeric($token) && count( $args ) > $token) {
				$val = $args[(int)$token];
			} else {
				$val = RunnerContext::getValue($token);
			}

			/**
			 * Don't ever dare to alter this code!
			 * Everything outside quotes must be converted to number to avoid SQL injection
			 */
			if ($conn->positionQuoted($sql, $offset))
				$repl["insert"] = $conn->addSlashes($val);
			else
				$repl["insert"] = DB::prepareNumberValue($val);

			$replacements[] = $repl;
		}

		//	do replacements
		return RunnerContext::doReplacements( $sql, $replacements );
	}

	/**
	 *	@return Array
	 */
	public static function readSQLTokens( $sql )
	{
		$arr = DB::scanTokenString( $sql );
		return $arr["tokens"];
	}

	/**
	 *	@return Array
	 */
	public static function readMasterTokens( $sql )
	{
		$masterTokens = array();

		$allTokens = DB::readSQLTokens( $sql );
		foreach ( $allTokens as $key => $token )
		{
			$dotPos = strpos(  $token, "." );
			if( $dotPos !== FALSE && strtolower( substr( $token, 0, $dotPos ) ) == "master")
			{
				$masterTokens[] = $token;
			}
		}

		return $masterTokens;
	}

	/**
	 *	Scans SQL string, finds all tokens. Returns three arrays - 'tokens', 'matches' and 'offsets'
	 *  Offsets are positions of corresponding 'matches' items in the string
	 *  Example:
	 *  insert into table values (':aaa', :old.bbb, ':{master.order date}')
	 *  tokens: ["aaa", "old.bbb", "master.order date"]
	 *  matches: [":aaa", ":old.bbb", ":{master.order date}"]
	 *  offsets: [28, 35, 46]
	 *
	 *	Exceptions for tokens without {}
	 *	1. shouldn't start with number
	*		:62aaa
	 *	2. shouldn't follow letter
	 *		x:aaa
	 *	3. shouldn't follow :
	 *		::aaa
	 *
 	 *	@return Array [ "tokens" => Array, "matches" => Array, "offsets" => Array ]
	 */
	public static function scanTokenString($sql)
	{
		$tokens = array();
		$offsets = array();
		$matches = array();

		//	match aaa, old.bbb, master.order date from:
		//	insert into table values (':aaa', :old.bbb, ':{master.order date}')

		$pattern = '/(?:[^\w\:]|^)(\:([a-zA-Z_]{1}[\w\.]*))|\:\{(.*?)\}|(?:[^\w\:]|^)(\:([1-9]+[0-9]*))/';


		$result = findMatches($pattern, $sql);
		foreach ($result as $m) {
			if ($m["submatches"][0] != "") {
				// first variant, no {}
				$matches[] = $m["submatches"][0];
				$tokens[] = $m["submatches"][1];
				$offsets[] = $m["offset"] + strpos($m["match"], $m["submatches"][0]);
			} else if ($m["submatches"][2] != "") {
				// second variant, in {}
				$matches[] = $m["match"];
				$tokens[] = $m["submatches"][2];
				$offsets[] = $m["offset"];
			} else if ($m["submatches"][3] != "") {
				// third variant, numeric like (:1, ':2')
				$matches[] = $m["submatches"][3];
				$tokens[] = $m["submatches"][4];
				$offsets[] = $m["offset"] + strpos($m["match"], $m["submatches"][3]);
			}
		}

		return array("tokens" => $tokens, "matches" => $matches, "offsets" => $offsets);
	}

	public static function scanNewTokenString($sql)
	{
		$tokens = array();
		$offsets = array();
		$matches = array();

		//	match aaa, old.bbb, master.order date from:
		//	insert into table values (':aaa', :old.bbb, ':{master.order date}')

		$pattern = "/\\\${[^\\s\{\\}]+}/";


		$result = findMatches($pattern, $sql);
		foreach ($result as $m) {
			$match = $m["match"];
			if ( $match != "" ) {
				$matches[] = $match;
				$tokens[] = substr( $match, 2, strlen( $match ) - 3 );
				$offsets[] = $m["offset"];
			}
		}

		return array("tokens" => $tokens, "matches" => $matches, "offsets" => $offsets);
	}


	public static function prepareNumberValue( $value )
	{
		$strvalue = str_replace( ",", ".", (string)$value );
		if( is_numeric($strvalue) )
			return $strvalue;
		return 0;
	}

	public static function Lookup( $sql ) {
		$result = DB::Query( $sql );
		if( !$result ) {
			return null;
		}
		$data = $result->fetchNumeric();
		if( !$data ) {
			return null;
		}
		return $data[0];
	}
	
	public static function DBLookup( $sql ) {
		return DB::Lookup( $sql );
	}

}

?>